import json
import os
import random
import time
from functools import wraps
from selenium import webdriver
from selenium.common.exceptions import InvalidArgumentException
from selenium.webdriver import Remote
from selenium.webdriver.chrome import options
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select, WebDriverWait


class ReuseChrome(Remote):
    def __init__(self, command_executor, session_id):
        self.r_session_id = session_id
        Remote.__init__(
            self, command_executor=command_executor, desired_capabilities={}
        )

    def start_session(self, capabilities, browser_profile=None):
        """
        重写start_session方法
        """
        if not isinstance(capabilities, dict):
            raise InvalidArgumentException("Capabilities must be a dictionary")
        if browser_profile:
            if "moz:firefoxOptions" in capabilities:
                capabilities["moz:firefoxOptions"]["profile"] = browser_profile.encoded
            else:
                capabilities.update({"firefox_profile": browser_profile.encoded})

        self.capabilities = options.Options().to_capabilities()
        self.session_id = self.r_session_id
        self.w3c = False


def retry(func, *, timeout=10):
    funcname = func.__name__
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        timeout = self.timeout
        for i in range(timeout):
            try:
                return func(self,*args, **kwargs)
            except:
                print(f"wait for {funcname} {args} {i}s")
                time.sleep(1)
        else:
            return func(self,*args, **kwargs)

    return wrapper


# class ProjectConfig:
#     driver = None


# project_config = ProjectConfig()
# project_config.timeout = 5
# project_config.ignore_errors = False


class Spider:
    def __init__(self):
        self.timeout = 10

    def connect_driver(self, executor_url, session_id):
        """
        重新连接已经打开的浏览器实例
        """
        self.driver = ReuseChrome(command_executor=executor_url, session_id=session_id)
        # return driver

    def init_driver(self):
        options = webdriver.ChromeOptions()
        options.add_argument("--disable-blink-features=AutomationControlled")
        options.add_experimental_option("w3c", False)
        driver = webdriver.Chrome(options=options)
        # driver = webdriver.Chrome(options=options, executable_path="./chromedriver.exe")
        executor_url = driver.command_executor._url
        session_id = driver.session_id
        print(executor_url, session_id)
        self.driver = driver
        return driver

    @retry
    def click(self, locator, index=0):
        self.driver.find_elements(By.CSS_SELECTOR, locator)[index].click()

    @retry
    def select(self, locator, value):
        select_elem = Select(self.driver.find_element(By.CSS_SELECTOR, locator))
        select_elem.select_by_value(value)

    @retry
    def send_keys(self, locator, value):
        self.driver.find_element(By.CSS_SELECTOR, locator).send_keys(value)


# driver = None

